﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using TwinCAT.Ads;
using TwinCAT.Scope2.Communications;
using TwinCAT.Scope2.View.ScopeViewControlLib;

namespace CS_Scope3
{
    class XYChart
    {
        //Ads端口号
        private int AdsPort { get; set; }
        //构造函数
        public XYChart(int AdsPort)
        {
            this.AdsPort = AdsPort;
        }
        //切换Scope的运行状态
        private void ChangeStateToConfig(ref ScopeViewControl SVC1)
        {
            //如果Scope运行模式是记录模式，则将当前记录状态停止
            if (SVC1.State == ScopeViewControlStates.RECORD)
            {
                SVC1.Operating.StopRecord();
            }
            if (SVC1.State == ScopeViewControlStates.REPLY)
            {
                SVC1.Operating.DisConnect(false);
            }
        }

        public bool AddChart(ref ScopeViewControl SVC1)
        {
            bool bReturn = false;
            try
            {
                ChangeStateToConfig(ref SVC1);
                ScopeViewControlChart chart = SVC1.NewChart();
                chart.ChartType = scopeDisplayModes.XY;
                SVC1.ViewDetailLevel = ScopeViewDetailLevel.Extended;
                chart.Style.DataPointToolTip = true;
                bReturn = true;
            }
            catch { }

            return bReturn;
        }
        public bool DeleteChart(ref ScopeViewControl SVC1, int ChartIndex)
        {
            bool bReturn = false;
            try
            {
                ChangeStateToConfig(ref SVC1);
                SVC1.DeleteChart(SVC1.Charts[ChartIndex]);
                bReturn = true;
            }
            catch { }
            return bReturn;
        }
        public bool AddAxis(ref ScopeViewControl SVC1, int ChartIndex)
        {
            bool bReturn = false;
            try
            {
                ChangeStateToConfig(ref SVC1);
                ScopeViewControlXYAxis xyAxis = (ScopeViewControlXYAxis)SVC1.Charts[ChartIndex].NewAxis();
                bReturn = true;
            }
            catch { }

            return bReturn;

        }
        public bool DeleteAxis(ref ScopeViewControl SVC1, int ChartIndex, int AxisIndex)
        {
            bool bReturn = false;
            try
            {
                ChangeStateToConfig(ref SVC1);
                SVC1.Charts[ChartIndex].DeleteAxis(SVC1.Charts[ChartIndex].Axes[AxisIndex]);
                bReturn = true;
            }
            catch { }

            return bReturn;

        }
        public bool AddChannel(ref ScopeViewControl SVC1, int ChartIndex, int AxisIndex)
        {
            bool bReturn = false;
            try
            {
                ChangeStateToConfig(ref SVC1);
                ScopeViewControlXYAxis xyAxis = (ScopeViewControlXYAxis)SVC1.Charts[ChartIndex].Axes[AxisIndex];
                ScopeViewControlXYChannel xychannel = xyAxis.NewChannelPair();
                xychannel.Acquisition.TargetPort = this.AdsPort;
                xychannel.XAcquisition.TargetPort = this.AdsPort;

                bReturn = true;
            }
            catch { }

            return bReturn;
        }
        public bool DeleteChannel(ref ScopeViewControl SVC1, int ChartIndex, int AxisIndex, int ChannelIndex)
        {
            bool bReturn = false;
            try
            {
                ChangeStateToConfig(ref SVC1);
                SVC1.Charts[ChartIndex].Axes[AxisIndex].DeleteChannel(SVC1.Charts[ChartIndex].Axes[AxisIndex].Channels[ChannelIndex]);
                bReturn = true;
            }
            catch { }

            return bReturn;
        }
        public bool ChartRename(ref ScopeViewControl SVC1, int ChartIndex, string ChartNewName)
        {
            bool bReturn = false;
            try
            {
                ChangeStateToConfig(ref SVC1);
                SVC1.Charts[ChartIndex].Name = ChartNewName;
                bReturn = true;
            }
            catch { }

            return bReturn;
        }
        public bool AxisRename(ref ScopeViewControl SVC1, int ChartIndex, int AxisIndex, string AxisNewName)
        {
            bool bReturn = false;
            try
            {
                ChangeStateToConfig(ref SVC1);
                SVC1.Charts[ChartIndex].Axes[AxisIndex].Name = AxisNewName;
                bReturn = true;
            }
            catch { }

            return bReturn;
        }
        public bool ChannelRename(ref ScopeViewControl SVC1, int ChartIndex, int AxisIndex, int ChannelIndex, string ChannelNewName)
        {
            bool bReturn = false;
            try
            {
                ChangeStateToConfig(ref SVC1);
                SVC1.Charts[ChartIndex].Axes[AxisIndex].Channels[ChannelIndex].Name = ChannelNewName;
                bReturn = true;
            }
            catch { }

            return bReturn;
        }

    }
}
