﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using TwinCAT.Ads;
using TwinCAT.Scope2.Communications;
using TwinCAT.Scope2.View.ScopeViewControlLib;
using System.IO;


namespace CS_Scope3
{
    public partial class TreeViewForm : Form
    {
        //树形节点选中时节点的顺序，和GlobalVar中数组顺序相反
        private int[] TreeViewSelectIndexArray1 = new int[4];
        //右键菜单
        private ContextMenuStrip RightClickMenu;

        private ArrayChart ArrayChart1 = new ArrayChart(851);
        private XYChart XYChart1 = new XYChart(851);
        private YTChart YTChart1 = new YTChart(851);
        private ChartCursor ChartCursor1 = new ChartCursor();
        
        public int PrtWidth = 0;
        public int PrtHeight = 0;


        private string NodeBeforeName = "";

        //托管==返回给父窗口的按钮相应=========================
        public event EventHandler TreeViewSelectSendTick;
        protected void TreeViewSelectOnTick(EventArgs e)
        {
            if (TreeViewSelectSendTick != null)
            {
                TreeViewSelectSendTick(this, e);
            }
        }
        public event EventHandler TreeViewChannelCrusorChangeSendTick;
        protected void TreeViewChannelCrusorChangeOnTick(EventArgs e)
        {
            if (TreeViewChannelCrusorChangeSendTick != null)
            {
                TreeViewChannelCrusorChangeSendTick(this, e);
            }
        }

        public TreeViewForm()
        {
            InitializeComponent();
            this.TopLevel = false;//取消界面的TopLevel，否则添加不到Panel中
            PrtWidth = this.Width;
            PrtHeight = this.Height;
        }

        //尺寸改变刷新控件
        public void TreeViewFrmSizeChange()
        {
            this.Left = 0;
            this.Top = 0;
            this.Width = PrtWidth;
            this.Height = PrtHeight;
            treeView1.Height = this.Height - label1.Bottom;
        }
        //主界面中Chart切换时刷新TreeView的选中
        public void TreeViewChartSelect(int index)
        {
            if (treeView1.Nodes[0].Nodes[index] != null)
            {
                treeView1.SelectedNode = treeView1.Nodes[0].Nodes[index];
                treeView1.SelectedNode.BackColor = Color.LightSteelBlue;
                treeView1.SelectedNode.ForeColor = Color.White;
            }
        }
        //节点选中
        //节点选中排序 递归
        private void GetNodeIndexRecursion(TreeNode Node1, int index)
        {
            TreeViewSelectIndexArray1[index] = Node1.Index;
            index++;
            if (Node1.Parent != null)
            {
                GetNodeIndexRecursion(Node1.Parent, index);
            }
        }

        //节点被选中后进行排序计算，刷新属性界面
        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (e.Node == null) return; //无节点

            if (treeView1.Nodes.Count > 0)
            {
                GlobalVar.TreeViewSelectLevel = treeView1.SelectedNode.Level;
                GlobalVar.TreeViewSelectName = treeView1.SelectedNode.Text;
            }
            else
            {
                GlobalVar.TreeViewSelectLevel = -1;
                GlobalVar.TreeViewSelectName = "Null";
                return;
            }
            //===========递归 将节点选中的顺序调整为正向顺序，没有选中的为-1=====
            for (int i = 0; i < TreeViewSelectIndexArray1.Length; i++)
            {
                TreeViewSelectIndexArray1[i] = -1;
                GlobalVar.TreeViewSelectIndexArray[i] = -1;
            }
            GetNodeIndexRecursion(treeView1.SelectedNode, 0);
            int StartIndex = -1;
            for (int i = 0; i < TreeViewSelectIndexArray1.Length; i++)
            {
                if (TreeViewSelectIndexArray1[i] != -1)
                {
                    StartIndex = StartIndex + 1;
                }
            }
            for (int i = 0; i <= StartIndex; i++)
            {
                GlobalVar.TreeViewSelectIndexArray[i] = TreeViewSelectIndexArray1[StartIndex - i];
            }
            //==========================          
            //判断是不是Cursor
            if (GlobalVar.TreeViewSelectIndexArray[1] >= 0 &&
                GlobalVar.TreeViewSelectIndexArray[2] ==GlobalVar.scopeViewControl1.Charts[GlobalVar.TreeViewSelectIndexArray[1]].Axes.Count)
            {
                GlobalVar.bTreeViewSelectIsCursor = true;
            }
            else
            { GlobalVar.bTreeViewSelectIsCursor = false; }

            //通知主界面节点被选中了
            TreeViewSelectOnTick(e);

        }

        //双击 节点为编辑模式
        private void treeView1_NodeMouseDoubleClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            try
            {
                NodeBeforeName = treeView1.SelectedNode.Text;
                treeView1.LabelEdit = true;
                treeView1.SelectedNode.BeginEdit();
            }
            catch { }
        }
        //编辑结束修改节点名称
        private void treeView1_AfterLabelEdit(object sender, NodeLabelEditEventArgs e)
        {
            try
            {
                this.treeView1.LabelEdit = false;
                string NewName = "";
                if (e.Label == null || e.Label == "")
                {
                    this.BeginInvoke(
                     new MethodInvoker(delegate
                     {
                         treeView1.SelectedNode.Text = NodeBeforeName;
                     }));

                    return;
                }
                NewName = e.Label;
                switch (GlobalVar.TreeViewSelectLevel)
                {
                    case 0:
                        YTChart1.ProjectRename(ref GlobalVar.scopeViewControl1, NewName);
                        break;
                    case 1:
                        YTChart1.ChartRename(ref GlobalVar.scopeViewControl1, GlobalVar.TreeViewSelectIndexArray[1], NewName);
                        break;
                    case 2:
                        if (!GlobalVar.bTreeViewSelectIsCursor)
                        {
                            YTChart1.AxisRename(ref GlobalVar.scopeViewControl1, GlobalVar.TreeViewSelectIndexArray[1],
                                             GlobalVar.TreeViewSelectIndexArray[2], NewName);
                        }
                        else
                        {
                            this.BeginInvoke(
                                 new MethodInvoker(delegate{treeView1.SelectedNode.Text = NodeBeforeName;}));
                        }

                        break;
                    case 3:
                        if (!GlobalVar.bTreeViewSelectIsCursor)
                        {
                            YTChart1.ChannelRename(ref GlobalVar.scopeViewControl1, GlobalVar.TreeViewSelectIndexArray[1],
                                                GlobalVar.TreeViewSelectIndexArray[2], GlobalVar.TreeViewSelectIndexArray[3], NewName);
                        }
                        else
                        {
                            ChartCursor1.CursorRename(ref GlobalVar.scopeViewControl1, GlobalVar.TreeViewSelectIndexArray[1],
                                                        GlobalVar.TreeViewSelectIndexArray[3], NewName);
                        }

                        break;
                    default: break;
                }
                TreeViewChannelCrusorChangeOnTick(e);
            }
            catch(Exception exp)
            { MessageBox.Show(exp.ToString()); }

        }
        //丢失鼠标焦点 节点颜色改变
        private void treeView1_MouseLeave(object sender, EventArgs e)
        {
            if (treeView1.SelectedNode != null)
            {

                treeView1.SelectedNode.BackColor = Color.LightSteelBlue;

                treeView1.SelectedNode.ForeColor = Color.White;
            }
        }
        //节点选中之前 节点颜色恢复
        private void treeView1_BeforeSelect(object sender, TreeViewCancelEventArgs e)
        {
            if (treeView1.SelectedNode != null)
            {
                treeView1.SelectedNode.BackColor = Color.Empty;

                treeView1.SelectedNode.ForeColor = Color.Black;
            }
        }
        //刷新TreeView节点
        public void TreeViewRefresh(ref ScopeViewControl SVC1)
        {
            treeView1.Nodes.Clear();
            treeView1.Nodes.Add(SVC1.Title );
            
            for (int i = 0; i < SVC1.Charts.Count; i++)
            {
                treeView1.Nodes[0].Nodes.Add(SVC1.Charts[i].Name);

                for (int j = 0; j < SVC1.Charts[i].Axes.Count; j++)
                {
                    treeView1.Nodes[0].Nodes[i].Nodes.Add(SVC1.Charts[i].Axes[j].Name);
                    for (int k = 0; k < SVC1.Charts[i].Axes[j].Channels.Count; k++)
                    {
                        treeView1.Nodes[0].Nodes[i].Nodes[j].Nodes.Add(SVC1.Charts[i].Axes[j].Channels[k].Name);
                    }
                }
                //添加完后节点添加Cursor
                treeView1.Nodes[0].Nodes[i].Nodes.Add("Cursor");
                for (int k = 0; k < SVC1.Charts[i].CursorModule.Cursors.Count; k++)
                {
                    treeView1.Nodes[0].Nodes[i].Nodes[treeView1.Nodes[0].Nodes[i].Nodes.Count - 1].Nodes.Add(
                       SVC1.Charts[i].CursorModule.Cursors[k].Style.Name);
                }

            }
            treeView1.Nodes[0].ExpandAll();



            // TreeViewSelectLevel = -1;
        }
        //鼠标点击   右键
        private void treeView1_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            if (e.Button != MouseButtons.Right) return;
            if (e.Node == null) return; //无节点
            //if (e.Node.Level == 0) return;//初始节点，不需要进行编辑
            treeView1.SelectedNode = e.Node;
            //显示右键菜单
            MenuOnRightMouse(e.Node.Level);
            RightClickMenu.Show(treeView1, e.X, e.Y);
        }
         //右键菜单设置
        private void MenuOnRightMouse(int Level)
        {
            try
            {
                RightClickMenu = new ContextMenuStrip();
                ToolStripMenuItem MenuAdd;
                ToolStripMenuItem MenuAddXY;
                ToolStripMenuItem MenuAddArray;
                ToolStripMenuItem MenuDelete;
                ToolStripMenuItem MenuAddXCursor;
                ToolStripMenuItem MenuAddYCursor;

                switch (Level)
                {
                    case 0:
                        MenuAdd = new ToolStripMenuItem("New YT Chart");
                        MenuAdd.Click += new EventHandler(MenuAddYT_Click);
                        RightClickMenu.Items.Add(MenuAdd);
                        MenuAddXY = new ToolStripMenuItem("New XY Chart");
                        MenuAddXY.Click += new EventHandler(MenuAddXY_Click);
                        RightClickMenu.Items.Add(MenuAddXY);
                        MenuAddArray = new ToolStripMenuItem("New Array Chart");
                        MenuAddArray.Click += new EventHandler(MenuAddArray_Click);
                        RightClickMenu.Items.Add(MenuAddArray);
                        break;

                    case 1:
                        MenuAdd = new ToolStripMenuItem("NewAxis");
                        MenuAdd.Click += new EventHandler(MenuAdd_Click);
                        RightClickMenu.Items.Add(MenuAdd);
                        MenuDelete = new ToolStripMenuItem("DeleteChart");
                        MenuDelete.Click += new EventHandler(MenuDelete_Click);
                        RightClickMenu.Items.Add(MenuDelete);
                        break;
                    case 2:
                        if (!GlobalVar.bTreeViewSelectIsCursor)
                        {
                            MenuAdd = new ToolStripMenuItem("NewChannel");
                            MenuAdd.Click += new EventHandler(MenuAdd_Click);
                            RightClickMenu.Items.Add(MenuAdd);
                            MenuDelete = new ToolStripMenuItem("DeleteAxis");
                            MenuDelete.Click += new EventHandler(MenuDelete_Click);
                            RightClickMenu.Items.Add(MenuDelete);
                        }
                        else
                        {
                            MenuAddXCursor = new ToolStripMenuItem("NewXCursor");
                            MenuAddXCursor.Click += new EventHandler(MenuAddXCursor_Click);
                            RightClickMenu.Items.Add(MenuAddXCursor);
                            MenuAddYCursor = new ToolStripMenuItem("NewYCursor");
                            MenuAddYCursor.Click += new EventHandler(MenuAddYCursor_Click);
                            RightClickMenu.Items.Add(MenuAddYCursor);
                        }
                        break;
                    case 3:
                        MenuDelete = new ToolStripMenuItem("Delete");
                        MenuDelete.Click += new EventHandler(MenuDelete_Click);
                        RightClickMenu.Items.Add(MenuDelete);
                        break;
                    default: break;

                }



            }
            catch (Exception exp)
            {
                MessageBox.Show(exp.ToString());
            }

        }
        private void MenuAddYT_Click(object sender, EventArgs e)
        {            
            GlobalVar.ChartModeSet = scopeDisplayModes.XT;
            if (YTChart1.AddChart(ref GlobalVar.scopeViewControl1))
            { TreeViewRefresh(ref GlobalVar.scopeViewControl1); }
            else
            { MessageBox.Show("Failed!"); }
           
        }
        private void MenuAddXY_Click(object sender, EventArgs e)
        {
            GlobalVar.ChartModeSet = scopeDisplayModes.XY;
            if (XYChart1.AddChart(ref GlobalVar.scopeViewControl1))
            { TreeViewRefresh(ref GlobalVar.scopeViewControl1); }
            else
            { MessageBox.Show("Failed!"); }
        }
        private void MenuAddArray_Click(object sender, EventArgs e)
        {
            GlobalVar.ChartModeSet = scopeDisplayModes.ARRAY;
            if (ArrayChart1.AddChart(ref GlobalVar.scopeViewControl1))
            { TreeViewRefresh(ref GlobalVar.scopeViewControl1); }
            else
            { MessageBox.Show("Failed!"); }
        }
        private void MenuAdd_Click(object sender, EventArgs e)
        {
            GlobalVar.ChartMode =GlobalVar.scopeViewControl1.Charts[GlobalVar.TreeViewSelectIndexArray[1]].ChartType;
            if (GlobalVar.TreeViewSelectLevel == 1)
            {
                if (GlobalVar.ChartMode == scopeDisplayModes.XT)
                {
                    if (!YTChart1.AddAxis(ref GlobalVar.scopeViewControl1, GlobalVar.TreeViewSelectIndexArray[1]))
                    { MessageBox.Show("Failed!"); }
                }
                if (GlobalVar.ChartMode == scopeDisplayModes.XY)
                {
                    if (!XYChart1.AddAxis(ref GlobalVar.scopeViewControl1, GlobalVar.TreeViewSelectIndexArray[1]))
                    { MessageBox.Show("Failed!"); }
                }
                if (GlobalVar.ChartMode == scopeDisplayModes.ARRAY)
                {
                    if (!ArrayChart1.AddAxis(ref GlobalVar.scopeViewControl1, GlobalVar.TreeViewSelectIndexArray[1]))
                    { MessageBox.Show("Failed!"); }
                }
            }
            if (GlobalVar.TreeViewSelectLevel == 2)
            {
                if (GlobalVar.ChartMode == scopeDisplayModes.XT)
                {
                    if (!YTChart1.AddChannel (ref GlobalVar.scopeViewControl1, GlobalVar.TreeViewSelectIndexArray[1], GlobalVar.TreeViewSelectIndexArray[2]))
                    { MessageBox.Show("Failed!"); }
                }
                if (GlobalVar.ChartMode == scopeDisplayModes.XY)
                {
                    if (!XYChart1.AddChannel(ref GlobalVar.scopeViewControl1, GlobalVar.TreeViewSelectIndexArray[1], GlobalVar.TreeViewSelectIndexArray[2]))
                    { MessageBox.Show("Failed!"); }
                }
                if (GlobalVar.ChartMode == scopeDisplayModes.ARRAY)
                {
                    if (!ArrayChart1.AddChannel(ref GlobalVar.scopeViewControl1, GlobalVar.TreeViewSelectIndexArray[1], GlobalVar.TreeViewSelectIndexArray[2]))
                    { MessageBox.Show("Failed!"); }
                }
                TreeViewChannelCrusorChangeOnTick(e);
            }

            TreeViewRefresh(ref GlobalVar.scopeViewControl1);
            
        }
        private void MenuDelete_Click(object sender, EventArgs e)
        {
            GlobalVar.ChartMode = GlobalVar.scopeViewControl1.Charts[GlobalVar.TreeViewSelectIndexArray[1]].ChartType;
            if (GlobalVar.TreeViewSelectLevel == 1)
            {
                if (GlobalVar.ChartMode == scopeDisplayModes.XT)
                {
                    if (!YTChart1.DeleteChart(ref GlobalVar.scopeViewControl1, GlobalVar.TreeViewSelectIndexArray[1]))
                    { MessageBox.Show("Failed!"); }
                }
                if (GlobalVar.ChartMode == scopeDisplayModes.XY)
                {
                    if (!XYChart1.DeleteChart(ref GlobalVar.scopeViewControl1, GlobalVar.TreeViewSelectIndexArray[1]))
                    { MessageBox.Show("Failed!"); }
                }
                if (GlobalVar.ChartMode == scopeDisplayModes.ARRAY)
                {
                    if (!ArrayChart1.DeleteChart(ref GlobalVar.scopeViewControl1, GlobalVar.TreeViewSelectIndexArray[1]))
                    { MessageBox.Show("Failed!"); }
                }
            }
            if (GlobalVar.TreeViewSelectLevel == 2)
            {
                if (GlobalVar.ChartMode == scopeDisplayModes.XT)
                {
                    if (!YTChart1.DeleteAxis (ref GlobalVar.scopeViewControl1, GlobalVar.TreeViewSelectIndexArray[1], GlobalVar.TreeViewSelectIndexArray[2]))
                    { MessageBox.Show("Failed!"); }
                }
                if (GlobalVar.ChartMode == scopeDisplayModes.XY)
                {
                    if (!XYChart1.DeleteAxis(ref GlobalVar.scopeViewControl1, GlobalVar.TreeViewSelectIndexArray[1], GlobalVar.TreeViewSelectIndexArray[2]))
                    { MessageBox.Show("Failed!"); }
                }
                if (GlobalVar.ChartMode == scopeDisplayModes.ARRAY)
                {
                    if (!ArrayChart1.DeleteAxis(ref GlobalVar.scopeViewControl1, GlobalVar.TreeViewSelectIndexArray[1], GlobalVar.TreeViewSelectIndexArray[2]))
                    { MessageBox.Show("Failed!"); }
                }
            }
            if (GlobalVar.TreeViewSelectLevel == 3 && !GlobalVar.bTreeViewSelectIsCursor)
            {
                if (GlobalVar.ChartMode == scopeDisplayModes.XT)
                {
                    if (!YTChart1.DeleteChannel(ref GlobalVar.scopeViewControl1, GlobalVar.TreeViewSelectIndexArray[1],
                        GlobalVar.TreeViewSelectIndexArray[2], GlobalVar.TreeViewSelectIndexArray[3]))
                    { MessageBox.Show("Failed!"); }
                }
                if (GlobalVar.ChartMode == scopeDisplayModes.XY)
                {
                    if (!XYChart1.DeleteChannel(ref GlobalVar.scopeViewControl1, GlobalVar.TreeViewSelectIndexArray[1],
                         GlobalVar.TreeViewSelectIndexArray[2], GlobalVar.TreeViewSelectIndexArray[3]))
                    { MessageBox.Show("Failed!"); }
                }
                if (GlobalVar.ChartMode == scopeDisplayModes.ARRAY)
                {
                    if (!ArrayChart1.DeleteChannel(ref GlobalVar.scopeViewControl1, GlobalVar.TreeViewSelectIndexArray[1],
                         GlobalVar.TreeViewSelectIndexArray[2], GlobalVar.TreeViewSelectIndexArray[3]))
                    { MessageBox.Show("Failed!"); }
                }
                TreeViewChannelCrusorChangeOnTick(e);
            }

            if (GlobalVar.TreeViewSelectLevel == 3 && GlobalVar.bTreeViewSelectIsCursor)
            {
                if (!ChartCursor1.DeleteCursor(ref GlobalVar.scopeViewControl1, GlobalVar.TreeViewSelectIndexArray[1], GlobalVar.TreeViewSelectIndexArray[2]))
                { MessageBox.Show("Failed!"); }
                TreeViewChannelCrusorChangeOnTick(e);
            }
            TreeViewRefresh(ref GlobalVar.scopeViewControl1);
        }
        private void MenuAddXCursor_Click(object sender, EventArgs e)
        {
            if(ChartCursor1 .AddXCursor (ref GlobalVar.scopeViewControl1, GlobalVar.TreeViewSelectIndexArray[1]))
            {
                TreeViewRefresh(ref GlobalVar.scopeViewControl1);
                TreeViewChannelCrusorChangeOnTick(e);
            }
            else
            { MessageBox.Show("Failed!"); }
        }
        private void MenuAddYCursor_Click(object sender, EventArgs e)
        {
            if (ChartCursor1.AddYCursor(ref GlobalVar.scopeViewControl1, GlobalVar.TreeViewSelectIndexArray[1]))
            {
                TreeViewRefresh(ref GlobalVar.scopeViewControl1);
                TreeViewChannelCrusorChangeOnTick(e);
            }
            else
            { MessageBox.Show("Failed!"); }
        }
      
    }
}
